@@ -1,9 +1,11 @@
 import asyncio
+import datetime
 import functools
 import json
 import os
 import pandas as pd
 import requests
+import time
 import websockets
 from typing import Callable, Coroutine
 
@@ -193,7 +195,52 @@ class FinXRest:
                     return_df = pd.concat([return_df, this_frame], ignore_index=True)
         return return_df
 
+    def _calc_years_to_expiry(self, row):
+        print("row", row)
+        instrument_name = row['instrument_name']
+        years_to_expiry = 0.0
+        try:
+            day_value = instrument_name.split("-")[1][0:2]
+            month_value = instrument_name.split("-")[1][2:5]
+            year_value = instrument_name.split("-")[1][5:8]
+            day_int = int(day_value)
+        except:
+            day_value = instrument_name.split("-")[1][0:1]
+            month_value = instrument_name.split("-")[1][1:4]
+            year_value = instrument_name.split("-")[1][4:7]
+        date_string = year_value + '/' + month_value + '/' + day_value
+        expiry_unix = datetime.datetime.strptime(date_string, '%y/%b/%d').timestamp()
+        years_to_expiry = round((expiry_unix - time.time())) / 60 / 60 / 24 / 365
+        return years_to_expiry
+
+    def _parse_strike_price(self, row):
+        return row['instrument_name'].split("-")[2]
+
+    def _remove_incomplete_observation_sets(self,expiration_count, row):
+        if int(row.strike) < int(expiration_count):
+            return 'delete'
+        else:
+            return 'keep'
+
+    @Hybrid
+    async def prepare_vol_surface_inputs(self, filename, timeslice_datestamp, timeslice_width_seconds, underlying_symbol) -> str:
+        df = pd.DataFrame(await self.get_options_timeslice(timeslice_datestamp, timeslice_width_seconds, underlying_symbol))
+        # print('vol_surface_inputs raw:', df)
+        df['years_to_expiry'] = df.apply(lambda row: self._calc_years_to_expiry(row), axis=1).astype(str)
+        df['strike_price'] = df.apply(lambda row: self._parse_strike_price(row), axis=1)
+        output_df = df[['years_to_expiry', 'strike_price', 'mark_iv']]
+        renamed_df = output_df.rename(columns={"years_to_expiry": "expiration", "strike_price": "strike", "mark_iv": "price"})
+        # must be expiration,strike,price
+        #,redis_key,underlying_price,underlying_index,timestamp,state,settlement_price,open_interest,min_price,max_price,mark_price,mark_iv,last_price,interest_rate,instrument_name,index_price,estimated_delivery_price,bid_iv,best_bid_price,best_bid_amount,best_ask_price,best_ask_amount,ask_iv,vega,theta,rho,gamma,delta,volume,price_change,low,high
+        unique_strikes_df = renamed_df.drop_duplicates(subset=['expiration', 'strike'], keep='last')
+        sorted_df = unique_strikes_df.sort_values(by=['expiration', 'strike']).copy()
+        grouped_df = sorted_df.groupby(by=['expiration'], as_index=False).count()
+        expiration_count = len(grouped_df)
+        sorted_df['action'] = sorted_df.apply(lambda row: self._remove_incomplete_observation_sets(expiration_count, grouped_df.loc[grouped_df['expiration'] == row.expiration]), axis=1)
+        keeper_df = sorted_df.loc[sorted_df['action'] == 'keep'].copy()
+        keeper_df.to_csv(filename, columns=['expiration', 'strike', 'price'], index=False)
+        return filename
+
     @Hybrid
     async def connect(self):
         await self.__aenter__()
-
